from setuptools import setup, find_packages

setup(
    name='ytsum',
    version='0.1.5',
    packages=find_packages(),
    install_requires=[
        'torch==2.1.1+cu118',
        'requests==2.32.3',
        'pytube==15.0.0',
        'transformers==4.36.2',
        'langchain==0.2.3',
        'langchain_community==0.2.4',
        'langchain_together==0.1.3'        
    ],
    entry_points={
        'console_scripts': [],
    },
    author='Arihant Tripathi',
    author_email='tarihant2001@gmail.com',
    description='Summarize YouTube video instantly with the power of distill-whisper and Mixtral-8B',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/Siris2314/ytsum',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.7',
)



import torch
from .distill import load_speech_recognition_model, download_audio_from_youtube
from langchain.prompts import PromptTemplate
from langchain.chains import RetrievalQA, LLMChain
from langchain_community.embeddings import HuggingFaceBgeEmbeddings
from langchain_community.document_loaders import TextLoader
from langchain.text_splitter import RecursiveCharacterTextSplitter
from langchain_community.vectorstores import FAISS
from langchain.schema.runnable import RunnablePassthrough
from langchain_together import ChatTogether
import textwrap
import os

together_api_key = None

def set_together_api_key(api_key):
    global together_api_key
    together_api_key = api_key

def load_transcript(url):
    whisper = load_speech_recognition_model()
    audio_file = download_audio_from_youtube(url)
    transcription = whisper(audio_file, chunk_length_s=30, stride_length_s=5, batch_size=8)
    with open("new_transcript.txt", "w") as f:
        f.write(transcription["text"])

    if os.path.exists(audio_file):
        os.remove(audio_file)
        print(f"File '{audio_file}' deleted successfully.")
    return "new_transcript.txt"

def wrap_text_preserve_newlines(text, width=110):
    lines = text.split('\n')
    wrapped_lines = [textwrap.fill(line, width=width) for line in lines]
    wrapped_text = '\n'.join(wrapped_lines)
    return wrapped_text

def process_llm_response(llm_response):
    response_text = wrap_text_preserve_newlines(llm_response['text'])
    sources_list = [source.metadata['source'] for source in llm_response['context']]
    return {"answer": response_text, "sources": sources_list}

def get_prompt(instruction, sys_prompt):
    B_INST, E_INST = "[INST]", "[/INST]"
    B_SYS, E_SYS = "<<SYS>>\n", "\n<</SYS>>\n\n"
    system_prompt = B_SYS + sys_prompt + E_SYS
    template = B_INST + system_prompt + instruction + E_INST
    return template

def answer_youtube_question(youtube_url, query):
    if together_api_key is None:
        raise ValueError("Together AI API key is not set. Please set it using the 'set_together_api_key' function.")

    # Load transcript
    transcript_path = load_transcript(youtube_url)

    # Load documents
    loader = TextLoader(transcript_path)
    documents = loader.load()

    # Split documents into chunks
    text_splitter = RecursiveCharacterTextSplitter(chunk_size=1000, chunk_overlap=200)
    texts = text_splitter.split_documents(documents)

    # Load embeddings
    embeddings = HuggingFaceBgeEmbeddings(model_name="BAAI/bge-large-en-v1.5", model_kwargs={'device': torch.device('cuda' if torch.cuda.is_available() else 'cpu')}, encode_kwargs={'normalize_embeddings': True})

    # Create FAISS database
    db = FAISS.from_documents(texts, embeddings)

    # Load LLM model
    llm = ChatTogether(model="NousResearch/Nous-Hermes-2-Mixtral-8x7B-SFT", max_tokens=2048, together_api_key=together_api_key)

    # Create prompt
    instruction = "Given the context that has been provided. \n {context}, Answer the following question: \n{question}"
    sys_prompt = """You are an expert in YouTube video question and answering.
    You will be given context to answer from. Answer the questions with as much detail as possible and only in paragraphs.
    In case you do not know the answer, you can say 'I don't know' or 'I don't understand'.
    In all other cases provide an answer to the best of your ability."""
    prompt_template = get_prompt(instruction, sys_prompt)

    # Create retrieval chain
    retriever = db.as_retriever()
    template = PromptTemplate(template=prompt_template, input_variables=['context', 'question'])
    llm_chain = LLMChain(llm=llm, prompt=template)
    rag_chain = {"context": retriever, "question": RunnablePassthrough()} | llm_chain

    # Process query
    ans = rag_chain.invoke(query)
    return process_llm_response(ans)



import torch
from transformers import AutoModelForSpeechSeq2Seq, AutoProcessor, pipeline
from pytube import YouTube


def download_audio_from_youtube(url: str, video_name = "yes") -> str:
    """Downloads audio from a YouTube video.

    Args:
        url (str): YouTube video URL.
        video_name (str): Desired name for the downloaded audio file, default name yes

    Returns:
        str: Path to the downloaded audio file.
    """

    if(len(url) == 0):
        raise ValueError("No Video URL specified")

    video_url = YouTube(url)
    video = video_url.streams.filter(only_audio=True).first()
    filename = f"{video_name}.mp3"
    video.download(filename=filename)
    return filename


def load_speech_recognition_model(model_id="distil-whisper/distil-large-v3") -> pipeline:
    """Loads a speech recognition pipeline model.

    Args:
        model_id (str): Identifier of the model to load. Default "distil-whisper/distil-large-v2"

    Returns:
        pipeline: The loaded speech recognition pipeline.
    """

    device = "cuda" if torch.cuda.is_available() else "cpu"
    torch_dtype = torch.float16 if torch.cuda.is_available() else torch.float32

    model = AutoModelForSpeechSeq2Seq.from_pretrained(
        model_id, torch_dtype=torch_dtype, low_cpu_mem_usage=True, use_safetensors=True
    )
    model.to(device)

    processor = AutoProcessor.from_pretrained(model_id)

    whisper = pipeline(
        "automatic-speech-recognition",
        model=model,
        tokenizer=processor.tokenizer,
        feature_extractor=processor.feature_extractor,
        max_new_tokens=128,
        torch_dtype=torch_dtype,
        device=device,
    )
    return whisper